import {
  CoreResource,
  CoreResourceReader,
  ReadOnlyMemoryStore,
} from "../../core";
import { PimDeleteAttribute } from "../operation";
import { executePimDeleteAttribute } from "./pim-delete-attribute-executor";
import * as PIM from "../pim-vocabulary";

test("Delete attribute.", async () => {
  const operation = new PimDeleteAttribute();
  operation.pimAttribute = "http://localhost/1";

  const before = {
    "http://schema": {
      iri: "http://schema",
      types: [PIM.SCHEMA],
      pimParts: ["http://localhost/1"],
    },
    "http://localhost/1": {
      iri: "http://localhost/1",
      types: [PIM.ATTRIBUTE],
    },
  };

  const actual = await executePimDeleteAttribute(
    wrapResourcesWithReader(before),
    undefined,
    operation
  );

  expect(actual.failed).toBeFalsy();
  expect(actual.created).toEqual({});
  expect(actual.changed).toEqual({
    "http://schema": {
      iri: "http://schema",
      types: [PIM.SCHEMA],
      pimParts: [],
    },
  });
  expect(actual.deleted.sort()).toEqual(["http://localhost/1"].sort());
});

function wrapResourcesWithReader(resources: {
  [iri: string]: CoreResource;
}): CoreResourceReader {
  return ReadOnlyMemoryStore.create(resources);
}
